﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using RodokmenDiplomka.Data;

namespace RodokmenDiplomka
{
    [Serializable]
    public class Family : IGedZaznam
    {
        enum vetev { nic, marr, div };
        private int id; //fam

        public int Id
        {
            get { return id; }
            set { id = value; }
        }
        private Individual husbandId;

        public Individual HusbandId
        {
            get { return husbandId; }
            set { husbandId = value; }
        }
        private Individual wifeId;

        public Individual WifeId
        {
            get { return wifeId; }
            set { wifeId = value; }
        }

        private List<Individual> children;

        public List<Individual> Children
        {
            get { return children; }
            set { children = value; }
        }

        //svatba
        private bool marriage;

        public bool Marriage
        {
            get { return marriage; }
            set { marriage = value; }
        }
        private Datum marriageDate = new Datum();

        public string MarriageDate
        {
            get { return marriageDate.getDatum(); }
            set { marriageDate.setDatum(value); }
        }
        private Place marriagePlace = new Place();

        public string MarriagePlace
        {
            get { return marriagePlace.CelyNazev; }
            set { marriagePlace.CelyNazev = value; }
        }
        //rozvod
        private bool div;

        public bool Div
        {
            get { return div; }
            set { div = value; }
        }
        private Place divPlace = new Place();

        public string DivPlace
        {
            get { return divPlace.CelyNazev; }
            set { divPlace.CelyNazev = value; }
        }
        private Datum divDate = new Datum();

        public string DivDate
        {
            get { return divDate.getDatum(); }
            set { divDate.setDatum(value); }
        }
        private List<string> notes;

        public List<string> Notes
        {
            get { return notes; }
            set { notes = value; }
        }

        public bool PresnostSvatby { get { return marriageDate.isPresne(); } set { marriageDate.setPresne(value); } }
        public bool PresnostRozvodu { get { return divDate.isPresne(); } set { divDate.setPresne(value); } }

        public string Svedci { get; set; }//neprovadim import export do ged jeste!!!!!

        private vetev v;

        public Family()
        {
            id = -1;
            v = vetev.nic;
            children = new List<Individual>();
            notes = new List<string>();
        }

        public void ZatridToken(GedToken t)
        {
            if (Regex.IsMatch(t.Typ, "^@"))
            {
                Id = zkusPrevestNaID(t.Typ);
                return;
            }
            switch (t.Typ)
            {
                case "FAMC":
                case "FAMS":
                    Id = zkusPrevestNaID(t.Pole);
                    break;
                case "HUSB":
                    HusbandId = new Individual();
                    HusbandId.ZatridToken(t);
                    break;
                case "WIFE":
                    WifeId = new Individual();
                    WifeId.ZatridToken(t);
                    break;
                case "NCHI":
                    break;
                case "MARR":
                    v = vetev.marr;
                    Marriage = true;
                    break;
                case "PLAC":
                    if (v == vetev.marr)
                    {
                        MarriagePlace = t.Pole;
                    }
                    else if (v == vetev.div)
                    {
                        DivPlace = t.Pole;
                    }
                    break;
                case "DATE":
                    if (v == vetev.marr)
                    {
                        MarriageDate = t.Pole;
                    }
                    else if (v == vetev.div)
                    {
                        DivDate = t.Pole;
                    }
                    break;
                case "CHIL":
                    Individual child = new Individual();
                    child.ZatridToken(t);
                    Children.Add(child);
                    break;
                case "DIV":
                    v = vetev.div;
                    Div = true;
                    break;
                /*case "PLAC":                    
                    break;
                case "DATE":
                    break;*/
                default:
                    throw new Exception("Chybné rozhodování family");
            }
        }

        private int zkusPrevestNaID(string p)
        {
            if (Regex.IsMatch(p, "^@"))
            {
                int pom = Int32.Parse(p.Substring(4, p.Length - 5));
                return pom;
            }
            return -1;
        }


        public void Exportovat(System.IO.StreamWriter sw)
        {
            if (id >= 0) sw.WriteLine("0 @ROD" + Id + "@ FAM");
            if (HusbandId != null && HusbandId.Id >= 0) sw.WriteLine("1 HUSB @CL" + HusbandId.Id + "@");
            if (WifeId != null && WifeId.Id >= 0) sw.WriteLine("1 WIFE @CL" + WifeId.Id + "@");
            foreach (Individual item in Children)
            {
                sw.WriteLine("1 CHIL @CL" + item.Id + "@");
            }
            sw.WriteLine("1 NCHI " + Children.Count);
            if (Marriage)
            {
                sw.WriteLine("1 MARR ");
                if (!String.IsNullOrEmpty(MarriageDate))
                {
                    //sw.WriteLine("2 DATE " + MarriageDate);
                    sw.Write("2 ");
                    marriageDate.Exportovat(sw);
                }
                if (!String.IsNullOrEmpty(MarriagePlace))
                {
                    //sw.WriteLine("2 PLAC " + MarriagePlace);
                    sw.Write("2 ");
                    marriagePlace.Exportovat(sw);
                }
            }
            if (Div)
            {
                sw.WriteLine("1 DIV ");
                if (!String.IsNullOrEmpty(DivDate)){
                    //sw.WriteLine("2 DATE " + DivDate);
                    sw.Write("2 ");
                    divDate.Exportovat(sw);
                }
                if (!String.IsNullOrEmpty(DivPlace))
                {
                    //sw.WriteLine("2 PLAC " + DivPlace);
                    sw.Write("2 ");
                    divPlace.Exportovat(sw);
                }
            }
        }
    }
}
