﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using RodokmenDiplomka.Data;

namespace RodokmenDiplomka
{
    [Serializable]
    public class Individual : IGedZaznam
    {
        enum vetev { nic, buri, resi, died, birt };
        private int id; //INDI
        public bool Uzamceno { get; set; }
        public int Id
        {
            get { return id; }
            set { id = value; }
        }
        private string firstname;

        public string Firstname
        {
            get { return firstname; }
            set { firstname = value; }
        }
        private string surname;

        public string Surname
        {
            get { return surname; }
            set { surname = value; }
        }
        private string suffix;

        public string Suffix
        {
            get { return suffix; }
            set { suffix = value; }
        }
        private string maridName;

        public string MaridName
        {
            get { return maridName; }
            set { maridName = value; }
        }
        private string sex;

        public string Sex
        {
            get { return sex; }
            set { sex = value; }
        }
        private string birt;

        public string Birt
        {
            get { return birt; }
            set { birt = (value); }
        }
        private Datum birthDate = new Datum();

        public string BirthDate
        {
            get { return birthDate.getDatum(); }
            set { birthDate.setDatum(value); }
        }
        private Place birthPlace = new Place();

        public string BirthPlace
        {
            get { return birthPlace.CelyNazev; }
            set { birthPlace.CelyNazev = value; }
        }
        private string national;

        public string National
        {
            get { return national; }
            set { national = value; }
        }
        private string religion;

        public string Religion
        {
            get { return religion; }
            set { religion = value; }
        }
        private string died;

        public string Died
        {
            get { return died; }
            set { died = value; }
        }
        private Datum diedDate = new Datum();

        public string DiedDate
        {
            get { return diedDate.getDatum(); }
            set { diedDate.setDatum(value); }
        }
        private Place diedPlace = new Place();

        public string DiedPlace
        {
            get { return diedPlace.CelyNazev; }
            set { diedPlace.CelyNazev = value; }
        }
        private string diedCause;

        public string DiedCause
        {
            get { return diedCause; }
            set { diedCause = value; }
        }
        private bool buri = false;

        public bool Buri
        {
            get { return buri; }
            set { buri = value; }
        }
        private Place buriPlace = new Place();

        public string BuriPlace
        {
            get { return buriPlace.CelyNazev; }
            set { buriPlace.CelyNazev = value; }
        }
        private List<string> occu;

        public List<string> Occu
        {
            get { return occu; }
            set { occu = value; }
        }
        private List<string> educ;

        public List<string> Educ
        {
            get { return educ; }
            set { educ = value; }
        }
        private bool resi = false;

        public bool Resi
        {
            get { return resi; }
            set { resi = value; }
        }

        public List<string> Tel { get; set; }
        public List<string> WWW { get; set; }
        public List<string> Email { get; set; }

        private Place resiPlace = new Place();

        public string ResiPlace
        {
            get { return resiPlace.CelyNazev; }
            set { resiPlace.CelyNazev = value; }
        }
        private List<string> resiAddr;

        public List<string> ResiAddr
        {
            get { return resiAddr; }
            set { resiAddr = value; }
        }
        private bool note = false;

        public bool Note
        {
            get { return note; }
            set { note = value; }
        }
        private List<string> notesCont;

        public List<string> NotesCont
        {
            get { return notesCont; }
            set { notesCont = value; }
        }
        private vetev v;
        private List<Family> parentFamilyId;

        public List<Family> ParentFamilyId
        {
            get { return parentFamilyId; }
            set { parentFamilyId = value; }
        }
        private List<Family> spouseFamilyId;

        public List<Family> SpouseFamilyId
        {
            get { return spouseFamilyId; }
            set { spouseFamilyId = value; }
        }

        private string addr;

        public string Addr
        {
            get { return addr; }
            set { addr = value; }
        }

        private List<ObjFile> soubory;

        public List<ObjFile> Soubory
        {
            get { return soubory; }
            set { soubory = value; }
        }

        public string TitulPred { get; set; }
        public string TitulZa { get; set; }

        public bool PresnostNarozeni { get { return birthDate.isPresne(); } set { birthDate.setPresne(value); } }
        public bool PresnostUmrti { get { return diedDate.isPresne(); } set { diedDate.setPresne(value); } }

        private ObjFile aktualniObjekt = null;



        //---------------------------------------------------------------------
        public Individual()
        {
            Uzamceno = false;
            id = -1;
            v = vetev.nic;
            occu = new List<string>();
            educ = new List<string>();
            resiAddr = new List<string>();
           
            parentFamilyId = new List<Family>();
            spouseFamilyId = new List<Family>();
            soubory = new List<ObjFile>();
            this.Sex = "F";

            Tel = new List<string>();
            WWW = new List<string>();
            Email = new List<string>();
            notesCont = new List<string>();
        }

        public void ZatridToken(GedToken t)
        {
            if (Regex.IsMatch(t.Typ, "^@"))
            {
                int pom = Int32.Parse(t.Typ.Substring(3, t.Typ.Length - 4));
                Id = pom;
                return;
            }

            switch (t.Typ)
            {
                case "WIFE":
                case "HUSB":
                case "CHIL":
                    Id = zkusPrevestNaID(t.Pole);
                    break;
                case "NAME":
                    string[] slova = t.Pole.Split('/');
                    if (slova.Length > 0) Firstname = slova[0];
                    if (slova.Length > 1) Surname = slova[1];

                    break;
                case "_MARNM":
                    MaridName = t.Pole;
                    break;
                case "SEX":
                    Sex = t.Pole;
                    break;
                case "BIRT":
                    Birt = t.Pole;
                    v = vetev.birt;
                    break;
                case "DATE":
                    if (v == vetev.birt)
                    {
                        BirthDate = t.Pole;
                    }
                    else if (v == vetev.died)
                    {
                        DiedDate = t.Pole;
                    }

                    break;
                case "PLAC":

                    if (v == vetev.birt)
                    {
                        birthPlace = new Place();
                        BirthPlace = t.Pole;
                    }
                    else if (v == vetev.died)
                    {
                        diedPlace = new Place();
                        DiedPlace = t.Pole;
                    }
                    else if (v == vetev.resi)
                    {
                        resiPlace = new Place();
                        ResiPlace = t.Pole;
                    }
                    else if (v == vetev.buri)
                    {
                        buriPlace = new Place();
                        BuriPlace = t.Pole;
                    }
                    else
                    {
                        throw new Exception("Chybné rozhodování individual");
                    }
                    break;
                case "DEAT":
                    Died = t.Pole;
                    v = vetev.died;
                    break;
                /*case "DATE":
                    diedDate = t.Pole;
                    break;*/
                /*case "PLAC":
                    diedPlace = t.Pole;
                    break;*/
                case "CAUS":
                    DiedCause = t.Pole;
                    break;
                case "BURI":
                    Buri = true;
                    v = vetev.buri;
                    break;
                /*case "PLAC":
                    buriPlace = t.Pole;
                    break;*/
                case "EDUC":
                    Educ.Add(t.Pole);
                    break;
                case "OCCU":
                    Occu.Add(t.Pole);
                    break;
                case "RESI":
                    Resi = true;
                    v = vetev.resi;
                    break;
                /*case "PLAC":
                    resiPlace = t.Pole;
                    break;*/
                case "NOTE":
                    Note = true;
                    break;
                case "CONT":
                    NotesCont.Add(t.Pole);
                    break;
                case "FAMC":
                    Family fams = new Family();
                    fams.ZatridToken(t);
                    ParentFamilyId.Add(fams);
                    break;
                case "FAMS":
                    Family famc = new Family();
                    famc.ZatridToken(t);
                    SpouseFamilyId.Add(famc);
                    break;
                case "ADDR":
                    if (v == vetev.resi)
                    {
                        ResiAddr.Add(t.Pole);
                    }
                    else
                    {
                        throw new Exception("Chybné rozhodování individual");
                    }
                    break;
                case "RELI":
                    Religion = t.Pole;
                    break;
                case "NATI":
                    National = t.Pole;
                    break;
                case "OBJE":
                    aktualniObjekt = new ObjFile();
                    Soubory.Add(aktualniObjekt);
                    break;
                case "FILE":
                case "FORM":
                case "_PRIM":
                case "TITL":
                case "_TYPE":
                    if (aktualniObjekt != null)
                    {
                        aktualniObjekt.ZatridToken(t);
                    }
                    break;
                case "NPFX":
                    TitulPred = t.Pole;
                    break;
                case "NSFX":
                    TitulZa = t.Pole;
                    break;
                case "PHON":
                    Tel.Add(t.Pole);
                    break;
                case "EMAIL":
                    Email.Add(t.Pole);
                    break;
                case "WWW":
                    WWW.Add(t.Pole);
                    break;
                default:
                    throw new Exception("Chybné rozhodování individual");
            }
        }

        private int zkusPrevestNaID(string p)
        {
            if (Regex.IsMatch(p, "^@"))
            {
                int pom = Int32.Parse(p.Substring(3, p.Length - 4));
                return pom;
            }
            return -1;
        }

        public void Exportovat(System.IO.StreamWriter sw)
        {
            if (id >= 0) sw.WriteLine("0 @CL" + Id + "@ INDI");
            if (!String.IsNullOrEmpty(Firstname)) sw.Write("1 NAME " + Firstname.Trim());
            sw.Write(" /");
            if (!String.IsNullOrEmpty(Surname)) { sw.Write(Surname); }
            sw.WriteLine("/");
            if (!String.IsNullOrEmpty(MaridName)) sw.WriteLine("2 _MARNM " + MaridName);
            foreach (ObjFile item in Soubory)
            {
                item.Exportovat(sw);
            }

            if (!String.IsNullOrEmpty(TitulPred)) sw.WriteLine("2 NPFX " + TitulPred);
            if (!String.IsNullOrEmpty(TitulZa)) sw.WriteLine("2 NSFX " + TitulZa);
            if (!String.IsNullOrEmpty(Sex)) sw.WriteLine("1 SEX " + Sex);
            if (!String.IsNullOrEmpty(Birt))
            {
                if (!String.IsNullOrEmpty(Birt)) sw.WriteLine("1 BIRT " + Birt);
                //if (!String.IsNullOrEmpty(BirthDate)) sw.WriteLine("2 DATE " + BirthDate);
                if (!String.IsNullOrEmpty(BirthDate))
                {
                    sw.Write("2 ");
                    birthDate.Exportovat(sw);
                }
                if (!String.IsNullOrEmpty(BirthPlace))
                {
                    //sw.WriteLine("2 PLAC " + BirthPlace);
                    sw.Write("2 ");
                    birthPlace.Exportovat(sw);
                }
            }
            if (!String.IsNullOrEmpty(National)) sw.WriteLine("1 NATI " + National);

            if (!String.IsNullOrEmpty(Died))
            {
                if (!String.IsNullOrEmpty(Died)) sw.WriteLine("1 DEAT " + Died);
                if (!String.IsNullOrEmpty(DiedDate))
                {
                    sw.Write("2 ");
                    diedDate.Exportovat(sw);
                }
                if (!String.IsNullOrEmpty(DiedPlace))
                {
                    //sw.WriteLine("2 PLAC " + DiedPlace);
                    sw.Write("2 ");
                    diedPlace.Exportovat(sw);
                }
                if (!String.IsNullOrEmpty(DiedCause)) sw.WriteLine("2 CAUS " + DiedCause);
            }
            if (!String.IsNullOrEmpty(BuriPlace))
            {
                sw.WriteLine("1 BURI");
                sw.WriteLine("2 PLAC " + BuriPlace);
            }
            foreach (string item in Educ)
            {
                sw.WriteLine("1 EDUC " + item);
            }

            foreach (string item in Occu)
            {
                sw.WriteLine("1 OCCU " + item);
            }
            if (!String.IsNullOrEmpty(Religion)) sw.WriteLine("1 RELI " + Religion);
            if (!String.IsNullOrEmpty(ResiPlace) || Tel.Count>0
                || Email.Count >0
                || Tel.Count >0
                || WWW.Count > 0)
            {
                sw.WriteLine("1 RESI");
                if (!String.IsNullOrEmpty(ResiPlace))
                {
                    if (!String.IsNullOrEmpty(ResiPlace)) sw.WriteLine("2 PLAC " + ResiPlace);
                }

                if (Tel.Count>0){
                    foreach (string t in Tel)
                    {
                        sw.WriteLine("2 PHON " + t.Trim());
                    }
                }
                if (Email.Count>0){
                    foreach (string em in Email)
                    {
                        sw.WriteLine("2 EMAIL " + em.Trim());
                    }
                }
                if (WWW.Count > 0) {
                    foreach (string w in WWW)
                    {
                        sw.WriteLine("2 WWW " + w.Trim());
                    }
                }
            }
            foreach (string item in ResiAddr)
            {
                sw.WriteLine("1 RESI");
                sw.WriteLine("2 ADDR " + item);
            }
            if (NotesCont.Count > 0) sw.WriteLine("1 NOTE");
            foreach (string item in NotesCont)
            {
                sw.WriteLine("2 CONT " + item);
            }
            foreach (Family item in SpouseFamilyId)
            {
                sw.WriteLine("1 FAMS @ROD" + item.Id + "@");
            }
            foreach (Family item in ParentFamilyId)
            {
                sw.WriteLine("1 FAMC @ROD" + item.Id + "@");
            }
        }
        public String dejCelePrijmeniJmeno()
        {
            if (String.IsNullOrEmpty(Surname) || String.IsNullOrWhiteSpace(Surname)) return MaridName + " " + Firstname;
            return Surname + " " + Firstname;

        }

        public String dejCelePrijmeniJmenoNarozeni()
        {
            if (String.IsNullOrEmpty(Surname) || String.IsNullOrWhiteSpace(Surname)) return MaridName + " " + Firstname + " (" + getRokNarozeni() + ")";
            return Surname + " " + Firstname + " (" + getRokNarozeni() + ")";

        }

        public String dejOtce()
        {
            String vysledek = "";
            if (parentFamilyId != null && parentFamilyId.Count > 0)
            {
                if (parentFamilyId[0].HusbandId != null)
                    vysledek = parentFamilyId[0].HusbandId.dejCelePrijmeniJmenoNarozeni();
            }
            return vysledek;
        }

        public String dejMatku()
        {
            String vysledek = "";
            if (parentFamilyId != null && parentFamilyId.Count > 0)
            {
                if (parentFamilyId[0].WifeId != null)
                    vysledek = parentFamilyId[0].WifeId.dejCelePrijmeniJmenoNarozeni();
            }
            return vysledek;
        }
        public String getRokNarozeni()
        {
            return birthDate.getRok();
        }

        public int dejPocetNadurovniRodice()
        {
            int max = 0;
            if (id < 0) return 0;
            if (parentFamilyId != null)
            {
                if (parentFamilyId.Count == 0)
                {
                    return 1;
                }
                if (parentFamilyId.Count > 1)
                {
                    return 1;
                }
                foreach (Family f in parentFamilyId)
                {
                    if (f.HusbandId != null && f.HusbandId.id >= 0)
                    {
                        max = Math.Max(max, f.HusbandId.dejPocetNadurovniRodice());
                    }
                    if (f.WifeId != null && f.WifeId.id >= 0)
                    {
                        max = Math.Max(max, f.WifeId.dejPocetNadurovniRodice());
                    }
                }
            }
            return max + 1;
        }

        public int dejSirkuNadurovniRodice()
        {
            int max = 1;
            if (parentFamilyId != null)
            {
                if (parentFamilyId.Count == 0)
                {
                    return 1;
                }
                foreach (Family f in parentFamilyId)
                {
                    if (f.HusbandId != null && f.HusbandId.id >= 0)
                    {
                        max = Math.Max(max, f.HusbandId.dejSirkuNadurovniRodice());
                    }
                    if (f.WifeId != null && f.WifeId.id >= 0)
                    {
                        max = Math.Max(max, f.WifeId.dejSirkuNadurovniRodice());
                    }
                }
            }
            return 2 * max;
        }
        /*internal int DejUroven()
        {
            if (listVLevo == null && listVPravo == null && tokenListu == null)
            {
                return 0;
            }
            else if (listVLevo == null && listVPravo == null && tokenListu != null)
            {
                return 1;
            }
            else if (listVLevo != null && listVPravo == null && tokenListu == null)
            {
                return ListVLevo.DejUroven() + 1;
            }
            else if (listVLevo == null && listVPravo != null && tokenListu == null)
            {
                return listVPravo.DejUroven() + 1;
            } if (listVLevo != null && listVPravo != null && tokenListu != null)
            {
                return Math.Max(listVPravo.DejUroven(), listVLevo.DejUroven()) + 1;
            }
            else
            {
                return 1;
                //throw new Exception("Chyba pri vytváření náhledu");
            }
        }*/
    }
}
