﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using RodokmenDiplomka.Data;

namespace RodokmenDiplomka
{
    [Serializable]
    public class Rodokmen
    {
        private Header hlavicka;
        private List<Subm> upravovatele;

        private Dictionary<int, Individual> osoby;
        int aktualniIndexOsoby = -1;
        //private Dictionary<int, int> osobyZmeneneIndexy;

        private Dictionary<int, Family> rodiny;
        private Dictionary<int, int> rodinyZmeneneIndexy;
        public bool IsPrecnenoDoKonce { get; set; }

        public Rodokmen(List<GedToken> pole)
        {
            IsPrecnenoDoKonce = false;
            hlavicka = new Header();
            upravovatele = new List<Subm>();
            osoby = new Dictionary<int, Individual>();
            //osobyZmeneneIndexy = new Dictionary<int, int>();
            rodiny = new Dictionary<int, Family>();
            rodinyZmeneneIndexy = new Dictionary<int, int>();
            IGedZaznam aktualniObjektPlneni = null;

            if (pole != null)
                for (int i = 0; i < pole.Count; i++)
                {
                    if (pole[i].Uroven == "0")
                    {
                        if (Regex.IsMatch(pole[i].Typ, "^HEAD$"))
                        {
                            aktualniObjektPlneni = hlavicka;
                        }
                        else if (Regex.IsMatch(pole[i].Typ, "^@SUBM") && pole[i].Pole.Trim() == "SUBM")
                        {
                            aktualniObjektPlneni = new Subm();
                            aktualniObjektPlneni.ZatridToken(pole[i]);
                            upravovatele.Add((Subm)aktualniObjektPlneni);

                            aktualniObjektPlneni.ZatridToken(pole[i]);
                        }
                        else if (Regex.IsMatch(pole[i].Typ, "^@CL") && pole[i].Pole.Trim() == "INDI")
                        {
                            aktualniObjektPlneni = new Individual();
                            aktualniObjektPlneni.ZatridToken(pole[i]);
                            int pom = ((Individual)aktualniObjektPlneni).Id;
                            aktualniIndexOsoby++;
                            /*if (aktualniIndexOsoby != pom)
                            {
                                osobyZmeneneIndexy.Add(pom, aktualniIndexOsoby);
                            }
                            osoby.Add(aktualniIndexOsoby, (Individual)aktualniObjektPlneni);*/
                            osoby.Add(pom, (Individual)aktualniObjektPlneni);
                        }
                        else if (Regex.IsMatch(pole[i].Typ, "^@ROD") && pole[i].Pole.Trim() == "FAM")
                        {
                            aktualniObjektPlneni = new Family();
                            aktualniObjektPlneni.ZatridToken(pole[i]);
                            int pom = ((Family)aktualniObjektPlneni).Id;
                            rodiny.Add(pom, (Family)aktualniObjektPlneni);
                        }
                        else if (Regex.IsMatch(pole[i].Typ, "^TRLR$"))
                        {
                            IsPrecnenoDoKonce = true;
                        }
                        else
                        {
                            throw new Exception("Chybné rozhodování rodokmen řádek: " + i + 1);
                        }
                    }
                    else if (aktualniObjektPlneni != null)
                    {
                        aktualniObjektPlneni.ZatridToken(pole[i]);
                    }
                    else
                    {
                        throw new Exception("Chybné rozhodování rodokmen");
                    }
                }
            sparujZaznamy();
        }

        private void sparujZaznamy()
        {
            foreach (Individual os in osoby.Values.ToList<Individual>())
            {
                for (int i = 0; i < os.ParentFamilyId.Count; i++)
                {
                    os.ParentFamilyId[i] = vratSpravnouRodinu(os.ParentFamilyId[i]);
                }
                for (int i = 0; i < os.SpouseFamilyId.Count; i++)
                {
                    os.SpouseFamilyId[i] = vratSpravnouRodinu(os.SpouseFamilyId[i]);
                }

            }
            foreach (Family fm in rodiny.Values.ToList<Family>())
            {
                fm.HusbandId = vratSpravnouOsobu(fm.HusbandId);
                fm.WifeId = vratSpravnouOsobu(fm.WifeId);

                for (int i = 0; i < fm.Children.Count; i++)
                {
                    fm.Children[i] = vratSpravnouOsobu(fm.Children[i]);
                }
            }
        }

        private Family vratSpravnouRodinu(Family pomF)
        {
            if (pomF == null) return pomF;

            pomF = rodiny[pomF.Id];
            /*foreach (Family f in rodiny.Values.ToList<Family>())
            {
                if (f.Id == pomF.Id) {
                    return f;
                }
            }*/

            return pomF;
        }

        private Individual vratSpravnouOsobu(Individual pomOS)
        {
            if (pomOS == null) return pomOS;

            int novyIndex = -1;
            /*try
            {
                novyIndex = osobyZmeneneIndexy[pomOS.Id];
            }
            catch (Exception e)
            {
                novyIndex = -1;
            }*/
            if (novyIndex >= 0)
            {
                pomOS = osoby[novyIndex];
            }
            else
            {
                pomOS = osoby[pomOS.Id];
            }
            return pomOS;
        }

        public void Exportovat(System.IO.StreamWriter sw)
        {
            hlavicka.Exportovat(sw);
            foreach (Subm item in upravovatele)
            {
                item.Exportovat(sw);
            }
            foreach (Individual item in osoby.Values.ToList<Individual>())
            {
                item.Exportovat(sw);
            }
            foreach (Family item in rodiny.Values.ToList<Family>())
            {
                item.Exportovat(sw);
            }
            sw.WriteLine("0 TRLR");
        }

        public IEnumerable<Individual> getOsoby()
        {
            return osoby.Values.ToList<Individual>();
        }

        public IEnumerable<Family> getRodiny()
        {
            return rodiny.Values.ToList<Family>();
        }

        internal Individual getOsoba(int index)
        {
            return osoby[index];
        }

        internal Individual vytvorOsobu()
        {
            Individual i = new Individual();
            int noveId = 0;
            while (osoby.ContainsKey(noveId))
            {
                noveId++;
            }
            i.Id = noveId;
            i.Surname = "Bezjmena";
            osoby.Add(noveId, i);
            return i;
        }

        internal bool VymazOsobu(Individual osoba)
        {
            if (osoba == null) return false;
            if (!osoby.ContainsKey(osoba.Id)) return false;

            osoby.Remove(osoba.Id);
            osoba = new Individual();//nefunguje musím opravdu všechno přenastavit v osobach rodinach atd

            return true;
        }

        internal bool UpdateOsoby(Individual aktualniOs, Individual novaOs)
        {
            if (aktualniOs == null || novaOs == null) return false;

            if (!osoby.ContainsKey(aktualniOs.Id)) return false;

            Individual os = osoby[aktualniOs.Id];

            os.Addr = novaOs.Addr;
            os.Birt = novaOs.Birt;
            os.BirthDate = novaOs.BirthDate;
            os.BirthPlace = novaOs.BirthPlace;
            os.Buri = novaOs.Buri;
            os.BuriPlace = novaOs.BuriPlace;
            os.Died = novaOs.Died;
            os.DiedCause = novaOs.DiedCause;
            os.DiedDate = novaOs.DiedDate;
            os.DiedPlace = novaOs.DiedPlace;
            os.Educ = novaOs.Educ;
            os.Email = novaOs.Email;
            os.Firstname = novaOs.Firstname;
            os.MaridName = novaOs.MaridName;
            os.National = novaOs.National;
            os.Note = novaOs.Note;
            os.NotesCont = novaOs.NotesCont;
            os.Occu = novaOs.Occu;
            os.ParentFamilyId = novaOs.ParentFamilyId;
            os.PresnostNarozeni = novaOs.PresnostNarozeni;
            os.PresnostUmrti = novaOs.PresnostUmrti;
            os.Religion = novaOs.Religion;
            os.Resi = novaOs.Resi;
            os.ResiAddr = novaOs.ResiAddr;
            os.ResiPlace = novaOs.ResiPlace;
            os.Sex = novaOs.Sex;
            os.Soubory = novaOs.Soubory;
            os.SpouseFamilyId = novaOs.SpouseFamilyId;
            os.Suffix = novaOs.Suffix;
            os.Surname = novaOs.Surname;
            os.Tel = novaOs.Tel;
            os.TitulPred = novaOs.TitulPred;
            os.TitulZa = novaOs.TitulZa;
            os.Uzamceno = novaOs.Uzamceno;
            os.WWW = novaOs.WWW;

            return true;
        }

        internal Dictionary<int, Individual> getOsobyDictionary()
        {
            return osoby;
        }

        internal Family VytvorRodinu()
        {
            Family f = new Family();
            int noveId = 0;
            while (rodiny.ContainsKey(noveId))
            {
                noveId++;
            }
            f.Id = noveId;
            rodiny.Add(noveId, f);
            return f;
        }

        internal bool OdeberRodinu(Family pomFam)
        {
            if (pomFam == null) return false;

            if (rodiny.ContainsKey(pomFam.Id))
            {
                rodiny.Remove(pomFam.Id);
                return true;
            }

            return false;

        }
    }
}
